unit mandbr;
{------------------------------------------------
   Mandelbrot set
   z = z * z + c
   (C) 1992,94,96 by Victor A.Voilevitch
   DRAW_METHODS: Old, Modern, Mixed, Black
-------------------------------------------------}

interface

uses
  Graphics;

const
  MAXC = 15;
  MAXM = 5;
type
  GoodColors = ARRAY [1..MAXC] of TColor;
  AvailMethods = ARRAY [1..MAXM] of String;
const
  ColWin : GoodColors = (        clBlack,    {1}
                                 clMaroon,   {2}
                                 clGreen,    {3}
                                 clOlive,    {4}
                                 clNavy,     {5}
                                 clPurple,   {6}
                                 clTeal,     {7}
                                 clGray,     {8}
                                 clSilver,   {9}
                                 clRed,      {10}
                                 clLime,     {11}
                                 clBlue,     {12}
                                 clFuchsia,  {13}
                                 clAqua,     {14}
                                 clWhite     {15}
                        );
  AvMet : AvailMethods = (       'Old',      {1}
                                 'Modern',   {2}
                                 'Mixed',    {3}
                                 'Black',    {4}
                                 'Green'     {5  $0000FF00 is pure green}
                         );
type

TMANDELBROT = class
private
    rb  : Double;       { start real  \                    }
    re  : Double;       { end real      \  area of         }
    ib  : Double;       { start image   /  complex numbers }
    ie  : Double;       { end image   /                    }
    sizx : Integer;     { x size of picture in pixels      }
    sizy : Integer;     { y size of picture in pixels      }
    art : String;       { draw method                      }
    backgr : TColor;    { background color                 }
    it     : Integer;   { total iterations                 }
    st     : Integer;   { first visible iteration          }
    ipc    : Integer;   { iterations per color             }
    {------------------------------------------------------}
    itr    : Integer;   { total iterations above threshold }
    rest   : Integer;   { iter mod MAXC                    }
    dit    : Integer;   { iter per color -> Old            }
    ai     : Integer;   { items          -> Mixed          }
    stc    : LongInt;   { start color                      }
public
    Constructor Create;
    Function GetPixel(X_,Y_ : Integer) : TColor;
    Function Init( rb_     : Double;
                   re_     : Double;
                   ib_     : Double;
                   ie_     : Double;
                   sizx_   : Integer;
                   sizy_   : Integer;
                   art_    : String;
                   backgr_ : TColor;
                   it_     : Integer;
                   st_     : Integer;
                   ipc_    : Integer
                 )                     : Boolean;
end;

function Is_AvMet(Str : String) : Boolean;
function Get_AvMets : String;

implementation

{================================================================}
Constructor TMANDELBROT.Create;
begin
  rb      := -2.25   ;
  re      := +0.25   ;
  ib      := -1.25   ;
  ie      := +1.25   ;
  sizx    := 100     ;
  sizy    := 100     ;
  art     := 'Black' ;
  backgr  := clBlack ;
  it      := 501;
  st      := 500;
  ipc     := 1;
  itr     := 1;
  rest    := 1;
  dit     := 1;
  ai      := 1;
  stc     := 1;
end;

{================================================================}
Function TMANDELBROT.Init(
                   rb_     : Double;
                   re_     : Double;
                   ib_     : Double;
                   ie_     : Double;
                   sizx_   : Integer;
                   sizy_   : Integer;
                   art_    : String;
                   backgr_ : TColor;
                   it_     : Integer;
                   st_     : Integer;
                   ipc_    : Integer
                         ) : Boolean;
begin
  rb      := rb_     ;
  re      := re_     ;
  ib      := ib_     ;
  ie      := ie_     ;
  sizx    := sizx_   ;
  sizy    := sizy_   ;
  art     := art_    ;
  backgr  := backgr_ ;
  it      := it_     ;
  st      := st_     ;
  ipc     := ipc_    ;

  if ( rb > re                   ) or
     ( ib > ie                   ) or
     ( sizx <= 0                 ) or
     ( sizy <= 0                 ) or
     ( it <= 0                   ) or
     ( st < 0                    ) or
     ( st >= it                  ) or
     ((ipc < 1) and (art='Mixed')) or
     ( not Is_AvMet(art)         ) or
     ( backgr < 0                )
  then Init := False
  else begin
    Init := True;
    {
                 |<--------------itr---------------->|
                 |                                   |
                 |<_dit_>|                           | \
                 |<--1--><--2-->..............<-MAXC>| / Old
                 |                                   |
     ------------=====================================x
     |           |                                   |
     1           st                                  it


    }
    itr  := it - st + 1;
    stc  := 1;
    rest := 0;
    if art = 'Old'
    then begin
      if itr<MAXC then stc  := MAXC - itr + 1;
      if itr>MAXC then rest := itr mod MAXC;
      dit := itr div MAXC;
      if dit=0 then dit := 1;
    end
    else if art = 'Modern'
    then begin
      if itr<MAXC then stc := MAXC - itr + 1
                  else stc := MAXC - (itr mod MAXC) + 1;
    end
    else if art = 'Mixed'
    then begin
      ai := itr div ipc;
      if (itr mod ipc)<>0 then Inc(ai);
      if ai<MAXC then stc := MAXC - ai + 1
                 else stc := MAXC - (ai mod MAXC) + 1;
    end
    else if art = 'Green'
    then begin
      if itr<$FF then stc := $FF - itr
                 else stc := 0;
      if itr>$FF then rest := itr mod $FF;
      dit := itr div $FF;
      if dit=0 then dit := 1;
      stc := $00000000 + (stc shl 8);
    end;
  end;
end;


{================================================================}
Function TMANDELBROT.GetPixel(X_,Y_ : Integer) : TColor;
var
  ci, cr : Double;                 { current point         }
  zr,zi  : Double;                 { current z             }
  iq,rq  : Double;                 { temporary             }
  setcol : TColor;                 { evaluated color       }
  lt : Integer;                    { current iteration     }
begin
  setcol := clBlack;

  ci := ib + Y_/sizy*(ie -ib);
  cr := rb + X_/sizx*(re -rb);
  zi := ci;
  zr := cr;
  iq := zi * zi;
  rq := zr * zr;
  lt := 0;
  repeat begin
    zi := 2 * zr * zi + ci;
    zr := rq - iq + cr;
    iq := zi * zi;
    rq := zr * zr;
    Inc(lt);
  end until (lt >= it) or  ((iq + rq) > 4);
  if lt < st              then setcol := 1
  else if( art='Black' )  then setcol := MAXC
  else if( art='Old' )    then begin
                            setcol := (lt-st-rest) div dit;
                            if setcol<0 then setcol := 0;
                            Inc(setcol,stc);
                          end
  else if( art='Green' )  then begin
                            setcol := (lt-st-rest) div dit;
                            if setcol<0 then setcol := 0;
                            setcol := setcol shl 8;
                            Inc(setcol,stc);
                          end
  else if( art='Modern' ) then begin
                            setcol := (stc + lt - st) mod MAXC;
                            if setcol=0 then setcol := MAXC;
                          end
  else if( art='Mixed' )  then begin
                            setcol := (stc + (lt-st) div ipc) mod MAXC;
                            if setcol=0 then setcol := MAXC;
                          end;
  if lt<st            then GetPixel := backgr
  else if art='Green' then GetPixel := setcol
  else                     GetPixel := ColWin[setcol];
end;

{================================================================}
function Is_AvMet(Str : String) : Boolean;
var
  i : Integer;
begin
  Is_AvMet := False;
  for i:= 1 to MAXM do
  if Str = AvMet[i]
  then begin
    Is_AvMet := True;
    Break;
  end;
end;

{================================================================}
function Get_AvMets : String;
var
  i : Integer;
  S : String;
begin
  S := 'Known METHODs are:';
  for i:= 1 to MAXM do
    S := S + ' ' + AvMet[i];
    S := S + '.';
  Get_AvMets := S;
end;



end.
